import click
import serial
import serial.tools.list_ports

from src.utilities import USBTalk

usb_device = click.make_pass_decorator(USBTalk, ensure=True)

@click.group(invoke_without_command=True)
@click.option('--port', '-p', default='', help='Manually select the device communications port')
@click.option('--baud_rate', '-b', default=112500, help='This is the serial baud rate')
@usb_device
def cli(usb_device, port, baud_rate):

  if port:
    usb_device.port = port
  else:
    for port in list(serial.tools.list_ports.comports()):
      if "Pico" in port.description:
        usb_device.port = port.device
        click.echo(f"Device found on port {usb_device.port}")

  if baud_rate:
    usb_device.baud_rate = baud_rate

  try:
    usb_device.serial_device = serial.Serial(port=usb_device.port, baudrate=usb_device.baud_rate, timeout=usb_device.serial_timeout)
  except:
    click.echo("Error communicating to device. Try selecting another port or verifying the USB connection.")
    exit()

  


@cli.command()
@click.option('--time', '-t', default='', help='Set the internal time')
@click.option('--stim_on', '-p', default=1, help='Set stimulation on')
@click.option('--stim_port', '-sp', default=8, help='Set the stimulation port')
@click.option('--tft_on', '-tft', default=1, help='Set the tft on/off')
@click.option('--broadcast_on', '-b', default=1, help='Set MQTT broadcasting on/off')
@click.option('--debug_on', '-b', default=1, help='Set debugging on/off')
@click.option('')
@usb_device
def set(usb_device):
  """
  configure variables on the device
  """
  click.echo(usb_device.write_read([0x05, 0x07]))


@cli.command()
@click.option('--time', '-t', default='', help='Get the internal time')
@click.option('--stim_on', '-p', default='', help='Get stimulation enable status')
@click.option('--stim_port', '-sp', default='', help='Get the enabled stimulation port')
@click.option('--tft_on', '-tft', default='', help='Get the tft status')
@click.option('--broadcast_on', '-b', default='', help='Get MQTT broadcasting status')
@click.option('--debug_on', '-b', default='', help='Get debug enable status')
@click.option('--devices', 'd', default='', help='Get the connected devices')
@usb_device
def get(usb_device):
  pass


@cli.command()
@usb_device
def broadcast(usb_device):
  """
  Broadcast commands from one device to another over MQTT
  """
  pass




def read_config():
  pass
  
def write_config():
  pass


def read_ports():
  pass

def write_ports():
  pass

def verify_net_comms():
  pass


@cli.result_callback()
@usb_device
def terminator(device, result, **kwargs):
  try:
    usb_device.serial_device.close()
  except:
    click.echo('Unable to close serial port.')  


import serial
import yaml
import click
import numpy as np
import time


class USBTalk:

  def __init__(self):

    with open('src/CommCodes.yaml', 'r') as file:
      self.comm_commands = yaml.safe_load(file)
      # for key in self.comm_commands.keys():
      #   self.comm_commands[key] = str(self.comm_commands[key])
    
    self.port = "/dev/cu.usbmodem1101"
    self.baud_rate = 112500
    self.serial_timeout = 0.5
    self.serial_device = None

    # Default
    # self.serial_device = serial.Serial(self.port, self.baud_rate, self.serial_timeout)
  


  def write_read(self, data:list):
    """
    Sends a list of readable commands to the device. The readable strings are converted to hex internally.
    data: Readable command strings in the form of a list
    """

    message = [self.comm_commands["start"]]
    message.append(np.random.randint(0, 255))
    message.extend(data)
    message.append(self.comm_commands["stop"])

    self.serial_device.write(bytearray(message))

    # receive the response
    try:
      output = list(self.serial_device.readline())
    except:
      click.echo('Unable to communicate (send or receive) data to device.')
      return None

    click.echo(output)

    return output
  
    
    return True

    # response = [float(x) for x in output.split(',') if x != '']

    # response.remove(actions.list['comm_termination'])

    # return response




from setuptools import setup, find_packages
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = fh.read()
setup(
    name = 'eopra_switcher',
    version = '0.0.1',
    author = 'Sean Boerhout',
    author_email = 'seanboe@mit.edu',
    license = 'MIT',
    description = 'CLI interface to the eOPRA port switcher',
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = 'https://github.com/seanboe/openServo',
    py_modules = ['main', 'src'],
    packages = find_packages(),
    install_requires = [requirements],
    python_requires='>=3.7',
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
    ],
    entry_points = '''
        [console_scripts]
        eopra_switcher=main:cli
    '''
)
